/*
 * TLED.c
 *
 * Created: 11.04.2017 10:58:29
 *  Author: dsosnowski
 */ 


#define F_CPU					8000000

//---includes-------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "disp.h"
#include "defs.h"
#include "dstemp.h"
#include "buttons.h"
#include "man.h"
#include "wdog.h"
#include "settings.h"

#define REL_PORT		PORTB
#define REL_DDR			DDRB
#define REL_PIN			5



//------------------------------------
struct{
	uint8_t sec;
	int16_t val;
	uint8_t status;
	uint8_t state;
} system;


void SystemInit(void);
void SystemSecProc(void);
uint8_t SystemSecGet(void);
void RelayProc(int16_t temp);


//------------------------------------
int main(void)
{
	WdtSet(7);
	SystemInit();
	ButtonInit();
	SettInit();
	DispInit();
	DsInit();

    while(1)
    {
		//-----------------------------------------------------
		ButtonProc();
		
		if (ButtonActionGet() > 0){
			ManKeyb(ButtonActionGet());
		}
				
		//-----------------------------------------------------
		if (SystemSecGet() > 0){
			DsProcess(TEMP_MAIN_CHANNEL);
			DsProcessNext();

			if (DsStatusGet(TEMP_MAIN_CHANNEL) == DS_READY) {
				system.val = DsTempGet(TEMP_MAIN_CHANNEL);
				RelayProc(system.val);
				system.status = 2;
			} else {
				if (DsStatusGet(TEMP_MAIN_CHANNEL) == DS_ERR) {
					REL_PORT &= ~(1<<REL_PIN);
					system.status = 0;
				}
			}
		}
		
		//-----------------------------------------------------
		if (ManModeGet() == MODE_STANDARD){
			if (system.status == 2){
				DispTemp(system.val);
				if (system.state > 0){
					DispDotSet(0);
				}
				system.status = 1;
			} 
			if (system.status == 0){			
				DispDigitsSet(DIGIT_MINUS_INDEX,DIGIT_MINUS_INDEX,DIGIT_MINUS_INDEX,DIGIT_NULL_VAL);			
			}			
		} else {
			ManDisp(ManModeGet());		
		}
	
		//-----------------------------------------------------
		SystemSecProc();
		_delay_ms(SYSTEM_LOOP_DELAY);
		WdtReset();
		
	}
}


//-------------------------------------
void SystemInit(void){
	REL_PORT &= ~(1<<REL_PIN);
	REL_DDR |= (1<<REL_PIN);	
}

//-------------------------------------
void SystemSecProc(void){
	static uint8_t persec;
	
	persec++;
	if (persec >= (950 / SYSTEM_LOOP_DELAY)){
		persec = 0;
		system.sec = 1;
	}
}

//-------------------------------------
uint8_t SystemSecGet(void){
	if (system.sec > 0){
		system.sec = 0;
		return 1;
	}
	return 0;
}

//------------------------------------
void RelayProc(int16_t temp){
	//static uint8_t state;
	
	if (SettValGet(0) <= SettValGet(1)){
		if (system.state == 0) {
			if (temp <= SettValGet(0)) {
				system.state = 1;
			}
		} else {
			if (temp >= SettValGet(1)) {
				system.state = 0;
			}			
		}		
	} else {
		if (system.state == 0) {
			if (temp >= SettValGet(0)) {
				system.state = 1;
			}
		} else {
			if (temp <= SettValGet(1)) {
				system.state = 0;
			}
		}		
	}
	
	if (system.state == 1) {
		REL_PORT |= (1<<REL_PIN);
	} else {
		REL_PORT &= ~(1<<REL_PIN);		
	}
}