/*
 * settings.c
 *
 * Created: 20.04.2017 11:25:21
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "defs.h"
#include "buttons.h"
#include "man.h"
#include "disp.h"
#include "dstemp.h"
#include "settings.h"


#define SETT_EEMEM_ADR		8
#define SETT_MAX_VAL		1250
#define SETT_MIN_VAL		-550
#define SETT_DEFAULT_ON		200
#define SETT_DEFAULT_OFF	250

void SettStore(char *pt, char adr, char len);
void SettRestore(char *pt, char adr, char len);
void EEwrite16(uint16_t address, int16_t data16);
int16_t EEread16(uint16_t address);
void EEwrite(uint16_t address, char data);
char EEread(uint16_t address);

//------------------------------------------------------
struct {
	uint8_t update;
	int16_t ton;
	int16_t toff;
} sett;


//------------------------------------------------------
void SettInit(void){
	SettRestore(&sett.ton, SETT_EEMEM_ADR, 4);
	
	if (((sett.ton % 10) > 0) || ((sett.toff % 10) > 0)){
		sett.ton = SETT_DEFAULT_ON;
		sett.toff = SETT_DEFAULT_OFF;		
	}
	
	SettChange(0, 0);
	SettChange(1, 0);
}

//------------------------------------------------------
int16_t *SettRefGet(uint8_t index){
	return ((&sett.ton) + index);
}

//------------------------------------------------------
int16_t SettValGet(uint8_t index){
	return *(SettRefGet(index));
}

//------------------------------------------------------
int16_t SettChange(uint8_t index, int16_t action){
	int16_t *temp;
	temp = SettRefGet(index);
	
	if (action != 0) sett.update = 1;
	
	*temp += action;
	if ((*temp % 10) > 0){
		*temp -= (*temp % 10);
	}
	
	if ((*temp) > SETT_MAX_VAL) {
		(*temp) = SETT_MAX_VAL;
	}
	if ((*temp) < SETT_MIN_VAL) {
		(*temp) = SETT_MIN_VAL;
	}
	return *temp;
}

//------------------------------------------------------
void SettSave(void){
	if (sett.update > 0){
		cli();
		SettStore(&sett.ton, SETT_EEMEM_ADR, 4);
		sett.update = 0;
		sei();
	}
}

//---------------------------------
void SettStore(char *pt, char adr, char len){
	while (len)
	{
		EEwrite(adr, *pt);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
void SettRestore(char *pt, char adr, char len){
	while (len)
	{
		*pt = EEread(adr);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
void EEwrite16(uint16_t address, int16_t data16){
	EEwrite(address, (char)(data16 & 0xFF));
	data16 >>= 8;
	EEwrite(address + 1, (char)(data16 & 0xFF));
}

//---------------------------------
int16_t EEread16(uint16_t address) {
	int16_t data16 = 0;
	data16 += EEread(address + 1);
	data16 <<= 8;
	data16 += EEread(address);
	return data16;
}

//---------------------------------
void EEwrite(uint16_t address, char data) {
	while(EECR & (1<<EEWE)); 		//Wait for completion of previous write
	EECR = 0;						//Set Programming mode

	EEAR = address;					//Set up address and data registers
	EEDR = data;

	EECR |= (1<<EEMWE);				//Write logical one to EEMPE
	EECR |= (1<<EEWE);				//Start eeprom write by setting EEPE
}

//---------------------------------
char EEread(uint16_t address) {
	while(EECR & (1<<EEWE));		//Wait for completion of previous write
	
	EEAR = address;					//Set up address register
	EECR |= (1<<EERE);				//Start eeprom read by writing EERE
	
	return EEDR;					//Return data from data register
}